#!/usr/bin/env python

#########################################################
# TCP Communication -- Server
#
# Name: Mengcheng Qi, Wendian Jiang
# NetID: mq58, wj225
# Lab Section: Wednesday 4:30 - 7:30
#
# Reference: https://wiki.python.org/moin/TcpCommunication
##########################################################

import socket
import RPi.GPIO as GPIO # for PWM control
import pigpio
import time as time # pull in the sleep function from time module


# set GPIO pins for PWM motor control
GPIO.setmode(GPIO.BCM)  # choose BCM or BOARD numbering schemes. I use BCM
GPIO.setup(18, GPIO.OUT) # set GPIO 18 as output for motor control
GPIO.setup(13, GPIO.OUT) # set GPIO 13 as output for motor control

pi_hw = pigpio.pi()   # connect to pi gpio daemon

def servo_control(servo, duty):
   if duty == 0:
      pi_hw.hardware_PWM(servo, 0, 0)
   else:
      n = 10000*duty
      pi_hw.hardware_PWM(servo, 50, n) # 50 Hz, n/1M duty cycle 

# control two wheels
def wheel_control(servo1, servo2, function, level):
    global lst_cmd
    
    if (level == 3):
        forwardL = 8.3
        forwardR = 8.3
        backwardL = 6.7
        backwardR = 6.9
    elif (level == 2): 
        forwardL = 7.8
        forwardR = 7.8
        backwardL = 7.2
        backwardR = 7.2
    elif (level == 1): 
        forwardL = 7.7
        forwardR = 7.7
        backwardL = 7.3
        backwardR = 7.3

    if function == 'forward':
        servo_control(servo1, backwardR)
        servo_control(servo2, forwardL)
    elif function == 'backward':
        servo_control(servo1, forwardR)
        servo_control(servo2, backwardL)
    elif function == 'left':
        servo_control(servo1, backwardR)
        servo_control(servo2, backwardL)
    elif function == 'right':
        servo_control(servo1, forwardR)
        servo_control(servo2, forwardL)
    elif function == 'stop':
        servo_control(servo1, 0)
        servo_control(servo2, 0)

    lst_cmd = function

motor1 = 18
motor2 = 13

pi_hw.hardware_PWM(motor1, 0, 0)   # 0 hz, 0 % duty cycle, fully stop
pi_hw.hardware_PWM(motor2, 0, 0)   # 0 hz, 0 % duty cycle, fully stop

TCP_IP   = ''
TCP_PORT = 5005
BUFFER_SIZE = 4

print 'waiting for connection ...'

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR,1)
s.bind((TCP_IP, TCP_PORT))
s.listen(1)

conn, addr = s.accept()
print 'Connection address:', addr

blevel = 2
flevel = 2
llevel = 2
rlevel = 2

try:
   # main loop
   while True:
      # receiving control command from client
      cmd = conn.recv(BUFFER_SIZE)
      print 'received cmd:', cmd # debug
      conn.send(cmd) # echo
      if not cmd : break

      # Control server based on the received command
      if cmd == 'back':
          wheel_control(motor1, motor2, 'backward', blevel)
      elif cmd == 'forw':
          wheel_control(motor1, motor2, 'forward', flevel)
      elif cmd == 'stop':
          wheel_control(motor1, motor2, 'stop', 2)
      elif cmd == 'righ':
          wheel_control(motor1, motor2, 'right', rlevel)
      elif cmd == 'left':
          wheel_control(motor1, motor2, 'left', llevel)
      elif cmd == 'fosl':
          flevel = 1
      elif cmd == 'fome':
          flevel = 2
      elif cmd == 'fofa':
          flevel = 3
      elif cmd == 'basl':
          blevel = 1
      elif cmd == 'bame':
          blevel = 2
      elif cmd == 'bafa':
          blevel = 3
      elif cmd == 'risl':
          rlevel = 1
      elif cmd == 'rime':
          rlevel = 2
      elif cmd == 'rifa':
          rlevel = 3
      elif cmd == 'lesl':
          llevel = 1
      elif cmd == 'leme':
          llevel = 2
      elif cmd == 'lefa':
          llevel = 3
      elif cmd == 'quit': 
          pi_hw.hardware_PWM(motor1, 0, 0)   # 0 hz, 0 % duty cycle, fully stop
          pi_hw.hardware_PWM(motor2, 0, 0)   # 0 hz, 0 % duty cycle, fully stop
          break
      elif cmd == 'test': 
          wheel_control(motor1, motor2, 'forward', 2)
          time.sleep(2)
          wheel_control(motor1, motor2, 'stop', 2)
          time.sleep(1)
          wheel_control(motor1, motor2, 'backward', 2)
          time.sleep(2)
          wheel_control(motor1, motor2, 'stop', 2)
          time.sleep(1)
          wheel_control(motor1, motor2, 'left', 2)
          time.sleep(1.5)
          wheel_control(motor1, motor2, 'stop', 2)
          time.sleep(1)
          wheel_control(motor1, motor2, 'right', 2)
          time.sleep(1.5)
          wheel_control(motor1, motor2, 'stop', 2)
          time.sleep(1)
      else:
          wheel_control(motor1, motor2, 'stop', 2)

   conn.close()
   pi_hw.stop() # close pi gpio DMA resources
   GPIO.cleanup()

except KeyboardInterrupt:
   conn.close()
   pi_hw.stop() # close pi gpio DMA resources
   GPIO.cleanup()

