#!/usr/bin/python

#########################################################
# Robot controller
#
# Name: Mengcheng Qi, Wendian Jiang
# NetID: mq58, wj225
# Lab Section: Wednesday 4:30 - 7:30
#
##########################################################

import pygame
from pygame.locals import *
import os # setup framebuffer
import RPi.GPIO as GPIO # quit button

from LIS3DH import LIS3DH
import time as time
import socket

cur_state = 0
pre_state = 0

GPIO.setmode(GPIO.BCM)
GPIO.setup(27, GPIO.IN, pull_up_down=GPIO.PUD_UP)

# set display shown on TFT and enable touchscreen
os.putenv('SDL_FBDEV', '/dev/fb1')

# IP address of the server
TCP_IP   = '10.148.4.93'
TCP_PORT = 5005
BUFFER_SIZE = 8

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR,1)
s.connect((TCP_IP, TCP_PORT))

# init pygame display and set mouse invisible
BLACK = (0,0,0)
WHITE = (255,255,255)
RED   = (255,0,0)
pygame.init()
pygame.mouse.set_visible(False)
lcd = pygame.display.set_mode((320, 240))
lcd.fill(BLACK)
pygame.display.update()

# functional diction for user interface display 
function_table = {'forw':'forward', 'back':'backward', 'left':'leftturn', 'righ':'rightturn'}
index_table    = {0:'Forward', 1:'Backward', 2:'Leftturn', 3:'Rightturn'}
speed_table    = {'Forward': 'Medium', 'Backward': 'Medium', 'Leftturn': 'Medium', 'Rightturn': 'Medium'}
left_table   = {'Medium':'Slow', 'Slow':'Fast', 'Fast':'Medium'}
right_table   = {'Fast':'Slow', 'Slow':'Medium', 'Medium':'Fast'}

# varibles / lists for user interface display
time_pin = 0
num = 1
record_first = 1
record_motion = []
record_time = []

# Pause interface display
def draw_pause():
    font_big = pygame.font.Font(None, 50)
    pygame.draw.rect(lcd, BLACK, (0,0,320,240), 0)
    text_surface = font_big.render('PAUSE', True, WHITE)
    rect = text_surface.get_rect(center= (180,120))
    lcd.blit(text_surface, rect)
    pygame.display.update()

# Motion log title display
def draw_log():
    font_big = pygame.font.Font(None, 26)
    pygame.draw.rect(lcd, BLACK, (0,0,320,240), 0)
    text_surface = font_big.render('MOTION LOG', True, WHITE)
    rect = text_surface.get_rect(center= (160,20))
    lcd.blit(text_surface, rect)
    pygame.display.update()

# Speed control title display
def draw_speed():
    font_big = pygame.font.Font(None, 26)
    pygame.draw.rect(lcd, BLACK, (0,0,320,240), 0)
    text_surface = font_big.render('SPEED CONTROL', True, WHITE)
    rect = text_surface.get_rect(center= (160,20))
    lcd.blit(text_surface, rect)
    pygame.display.update()

# log refresh and display
def resume_log():
    x, y = 50, 60
    font_big = pygame.font.Font(None, 21)
    dis_cnt = 0
    pygame.draw.rect(lcd, BLACK, (0,50,320,240), 0)
    
    # draw title first
    draw_log()

    # draw motion type
    for i in record_motion :
        if dis_cnt < 12 :
            text_surface = font_big.render('%s'%i, True, WHITE)
            rect = text_surface.get_rect(center= (x,y))
            y += 30
            lcd.blit(text_surface, rect)
        dis_cnt += 1
        if dis_cnt == 6:
            x, y = 210, 60
         
    x, y = 130, 60
    dis_cnt = 0

    # draw time consumption
    for i in record_time :
        if dis_cnt < 12 :
            text_surface = font_big.render('%s'%i, True, WHITE)
            rect = text_surface.get_rect(center= (x,y))
            y += 30
            lcd.blit(text_surface, rect)
        dis_cnt += 1
        if dis_cnt == 6:
            x, y = 290, 60
    pygame.display.update()

# speed control refresh and display
def resume_speed(option):
    lcd.fill(BLACK)

    # draw title first
    draw_speed()
    font_big = pygame.font.Font(None, 26)
    function_buttons = {'Forward':(110,50), 'Backward':(110,100),  'Leftturn':(110,150),  'Rightturn':(110,200) }
    red_y = 0

    # draw function buttons
    for k,v in function_buttons.items():
        if k == option :
            red_y = v[1]
            text_surface = font_big.render('%s'%k + '  :' , True, RED)
        else :
            text_surface = font_big.render('%s'%k + '  :' , True, WHITE)
        rect = text_surface.get_rect(center=v)
        lcd.blit(text_surface, rect)
    
    # draw speed level
    x, y = 220, 50
    for i in range(4):
        speed_table[index_table[i]]
        if y == red_y :
            text_surface = font_big.render( speed_table[index_table[i]], True, RED)
        else :
            text_surface = font_big.render( speed_table[index_table[i]], True, WHITE)
        rect = text_surface.get_rect(center=(x,y))
        lcd.blit(text_surface, rect)
        y += 50

    pygame.display.update()

#speed configuration message
def change_speed(option, speed):
    if option == 0:
        if   speed == 'Slow':   inpt = 'fosl'
        elif speed == 'Medium': inpt = 'fome'
        elif speed == 'Fast':   inpt = 'fofa'
    elif option == 1:
        if   speed == 'Slow':   inpt = 'basl'
        elif speed == 'Medium': inpt = 'bame'
        elif speed == 'Fast':   inpt = 'bafa'
    elif option == 2:
        if   speed == 'Slow':   inpt = 'lesl'
        elif speed == 'Medium': inpt = 'leme'
        elif speed == 'Fast':   inpt = 'lefa'
    elif option == 3:
        if   speed == 'Slow':   inpt = 'risl'
        elif speed == 'Medium': inpt = 'rime'
        elif speed == 'Fast':   inpt = 'rifa'
    s.send(inpt)
    
# motion history record    
def record(curr_time, function):
    global num, record_first, time_pin, record_motion, record_time 
    if record_first:
         record_first = 0
         time_pin = curr_time
         record_motion = [str(num) +') ' + function_table[function]] + record_motion
         num += 1

         x, y = 50, 60
         font_big = pygame.font.Font(None, 21)
         dis_cnt = 0
         dummy_first = 1

         pygame.draw.rect(lcd, BLACK, (0,50,320,240), 0)
         for i in record_motion :
             if dis_cnt < 12 :
                 if dummy_first :
                     text_surface = font_big.render('%s'%i, True, RED)
                     dummy_first = 0
                 else :
                     text_surface = font_big.render('%s'%i, True, WHITE)
                 rect = text_surface.get_rect(center= (x,y))
                 y += 30
                 lcd.blit(text_surface, rect)
             dis_cnt += 1
             if dis_cnt == 6:
                 x, y = 210, 60
         
         x, y = 130, 90
         dis_cnt = 0

         for i in record_time :
             if dis_cnt < 11 :
                 text_surface = font_big.render('%s'%i, True, WHITE)
                 rect = text_surface.get_rect(center= (x,y))
                 y += 30
                 lcd.blit(text_surface, rect)
             dis_cnt += 1
             if dis_cnt == 5:
                 x, y = 290, 60
         
    else:
         record_first = 1
         record_time = [str("%.2f" %(curr_time-time_pin)) + ' s'] + record_time
         
         font_big = pygame.font.Font(None, 21)
         text_surface = font_big.render('%s'%record_motion[0], True, WHITE)
         rect = text_surface.get_rect(center= (50,60))
         lcd.blit(text_surface, rect)
         text_surface = font_big.render('%s'%record_time[0], True, WHITE)
         rect = text_surface.get_rect(center= (130,60))
         lcd.blit(text_surface, rect)
         
         
    pygame.display.update()


if __name__ == '__main__':
    sensor = LIS3DH(debug=True)

    sensor.setClick(LIS3DH.CLK_ALL)
    sensor.setorient()

    trigger = 0
    counter = 0
    pre_click = 0
    click_cnt = 20

    option1 = 0
    option2 = 1

    draw_log()

    print "Starting stream"
    quit = False
    while not quit: 

        if  not GPIO.input(27) :
            print 'quit ...'
            quit = True
        
        if cur_state ==  0 :
            click_reg = sensor.getClick()
            click = click_reg & 0x30
            orient_reg = sensor.getorient()      
            orient = orient_reg & 0x40

            if click and not trigger :
                if click_reg & 0x10 : 
                    print " single click"
                    pre_click = 1
                    click_cnt = 40
                    
                if click_reg & 0x20 : 
                    print " double click"
                    pre_click = 0
                    pre_state = cur_state
                    cur_state = 1
                    resume_speed(index_table[0])
            # orientation detect
            elif not click and orient and counter < 3:
                counter  += 1
                # debounce pass
                if counter == 3 and not trigger :
                    print "send!"
                    if orient_reg == 0x48 :
                        inpt = 'forw'
                    elif orient_reg == 0x44 :
                        inpt = 'back'
                    elif orient_reg == 0x41 :
                        inpt = 'left'
                    elif orient_reg == 0x42 :
                        inpt = 'righ'
                    s.send(inpt)
                    trigger = 1
                    pre_click = 0
                    record(time.time(),inpt)
            elif not click and not orient and counter > 0:
                counter -= 1
                if counter == 0 and trigger :
                    print "stop"            
                    inpt = 'stop'
                    s.send(inpt)
                    trigger = 0
                    pre_click = 0
                    record(time.time(),inpt)
            if pre_click :
                click_cnt -= 1
                if  click_cnt == 0 :
                    pre_click = 0
                    pre_state = cur_state
                    cur_state = 2
                    draw_pause()

        elif cur_state ==  1 :
            click_reg = sensor.getClick()
            click = click_reg & 0x30
            orient_reg = sensor.getorient()      
            orient = orient_reg & 0x40
    
            if click and not trigger :
                if click_reg & 0x10 : 
                    print " single click"
                    pre_click = 1
                    click_cnt = 40
                if click_reg & 0x20 : 
                    print " double click"
                    record_motion = []
                    record_time = []
                    cur_state = 0
                    num = 1
                    resume_log()
            # orientation detect
            elif not click and orient and counter < 3:
                counter  += 1
                # debounce pass
                if counter == 3 and not trigger :
                    if orient_reg == 0x48 :
                        option1 -= 1
                        if option1 < 0 : option1 = 3 
                        resume_speed(index_table[option1])
                    elif orient_reg == 0x44 :
                        option1 += 1
                        if option1 > 3 : option1 = 0 
                        resume_speed(index_table[option1])
                    elif orient_reg == 0x41 :
                        speed_table[index_table[option1]] = left_table[speed_table[index_table[option1]]]
                        resume_speed(index_table[option1])
                        change_speed(option1, speed_table[index_table[option1]])
                    elif orient_reg == 0x42 :
                        speed_table[index_table[option1]] = right_table[speed_table[index_table[option1]]]
                        resume_speed(index_table[option1])
                        change_speed(option1, speed_table[index_table[option1]])
                    s.send(inpt)
                    trigger = 1
                    pre_click = 0
            elif not click and not orient and counter > 0:
                counter -= 1
                if counter == 0 and trigger :
                    inpt = 'stop'
                    s.send(inpt)
                    trigger = 0
                    pre_click = 0
            if pre_click :
                click_cnt -= 1
                if  click_cnt == 0 :
                    pre_click = 0
                    pre_state = cur_state
                    cur_state = 2
                    draw_pause()

        elif cur_state == 2 :
            click_reg = sensor.getClick()
            click = click_reg & 0x30
            if click and click_reg & 0x10 : 
                if pre_state == 0 : 
                    print " single click"
                    cur_state = 0
                    resume_log()
                elif pre_state == 1 :
                    print " single click"
                    cur_state = 1
                    resume_speed(index_table[option1])
                    
         
        time.sleep(0.05) 
